
package RightFieldsConvert;
use strict;
use Data::Dumper;

use base qw(MT::App::CMS);

use RightFields qw( load_plugindata save_plugindata config_key new_obj );

sub init {
	my $app = shift;
	$app->SUPER::init (@_) or return;
	$app->add_methods (
		'list_blogs' => \&list_blogs,
		'convert_confirm' => \&convert_confirm,
		'do_convert' => \&do_convert
	);
	$app->{'default_mode'} = 'list_blogs';
	$app->{'user_class'} = 'MT::Author';
	$app->{'plugin_template_path'} = 'tmpl';
	$app->{'requires_login'} = 1;
	$app;
}

sub build_page {
	my ($app, $page, $param) = @_;
	$app->add_breadcrumb('RightFields Data Converter');
	$app->SUPER::build_page($page, $param);
}

sub list_blogs {
	my ($app, $param, $errors) = @_;
	$app->add_breadcrumb('RightFields Data Converter');
	my $blogs = pd_blogs_for_author($app);
	my %ids = map { $_ => 1 } $app->param('id');
		# restore values
	for my $blog (@$blogs) {
		$blog->{'checked'} = $ids{$blog->{'id'}} ? 1 : 0;
		$blog->{'datasource'} = $app->param("datasource_$blog->{'id'}");
	}
	$param->{'rf_blog_loop'} = $blogs;
	$param->{'has_blogs'} = @$blogs ? 1 : 0;
	if ($errors) {
		$param->{'item_errors'} = 1;
		for my $blog(@$blogs) {
			$blog->{'item_error'} = $errors->{$blog->{'id'}};
		}
	}
	return $app->build_page('rf_sql_convert.tmpl', $param);
}

sub convert_confirm {
	my ($app) = @_;
	my @ids;
	if (!(@ids = $app->param('id'))) {
		return list_blogs($app, {
			'error' => q{You didn't specify any weblogs to convert.}
		});
	}
	my $errors = check_datasources($app, \@ids);
	if (%$errors) {
		return list_blogs($app, {}, $errors);
	}
	my @blogs = ();
	require MT::Blog;
	for my $id (@ids) {
		my $blog = MT::Blog->load($id);
		my $row = {
			'id' => $id,
			'name' => $blog->name,
			'datasource' => $app->param("datasource_$id")
		};
		push(@blogs, $row);
	}
	return $app->build_page('rf_sql_confirm.tmpl', { 
		'rf_blog_loop' => \@blogs
	});
}

sub do_convert {
	my ($app) = @_;
		# check these again on the off chance the system changed since
		# the convert_confirm call
	my @ids;
	if (!(@ids = $app->param('id'))) {
		return list_blogs($app, {
			'error' => q{You didn't specify any weblogs to convert.}
		});
	}
	my $errors = check_datasources($app, \@ids);
	if (%$errors) {
		return list_blogs($app, {}, $errors);
	}
	require MT::Blog;
	require MT::Entry;
	my @pd_delete_ids = ();
	my @added;
	for my $blog_id (@ids) {
		my $datasource = $app->param("datasource_$blog_id");
		my $cfg_key = config_key($blog_id, 'extra');
		my $cfg = load_plugindata($cfg_key);
		my $iter = MT::Entry->load_iter({ 'blog_id' => $blog_id });
		while (my $entry = $iter->()) {
			$cfg->{'datasource'} = '_pseudo';
			my $pd_obj = new_obj($cfg, $entry->id);
			next unless ($pd_obj->key);
			$cfg->{'datasource'} = $datasource;
			my $sql_obj = new_obj($cfg);
			$sql_obj->id($entry->id);
			my %vals = map { $_ => $pd_obj->$_ } keys %{$cfg->{'cols'}};
			$sql_obj->set_values(\%vals);
			$sql_obj->save || die $sql_obj->errstr;
			$pd_obj->remove || die $pd_obj->errstr;
		}
		$cfg->{'datasource'} = $datasource;
			# if columns in the table are not in the config, set them to Ignore
		my $db_defs = db_defs($datasource);
		for my $col (keys %$db_defs) {
			if (!$cfg->{'cols'}->{$col}) {
				$cfg->{'cols'}->{$col} = { 'sort' => 999, 'ignore' => 1 };
			}
		}
		my $data_key = config_key($blog_id, 'extra');
		save_plugindata($data_key, $cfg) || die "Couldn't save plugindata";
	}
	return $app->build_page('rf_sql_converted.tmpl');
}

sub check_datasources {
	my ($app, $ids) = @_;
	my %errors = ();
	for my $id (@$ids) {
		my $cfg_key = config_key($id, 'extra');
		my $cfg = load_plugindata($cfg_key);
			# this shouldn't happen, and if it does it won't show up
			# when the list reloads anyway
		next if ((!$cfg) || ($cfg->{'datasource'} ne '_pseudo'));
		my $datasource = $app->param("datasource_$id");
		if (!$datasource) {
			$errors{$id} = 'No SQL datasource specified';
			next;
		}
		my $db_defs = db_defs($datasource);
		if (!ref($db_defs)) {
			$errors{$id} = $db_defs;
			next;
		}
			# make sure columns in datasource exist corresponding to
			# all fields in PluginData
		my @missing;
		for my $field (keys %{$cfg->{'cols'}}) {
			if (!$db_defs->{$field}) {
				push(@missing, $field);
			}
		}
		if (@missing) {
			$errors{$id} = qq{The following columns are missing from the <b>mt_$datasource</b> table: <b>}
				. join(', ', map { "${datasource}_$_" } @missing) . '</b>';
			next;
		}
	}
	return \%errors;
}

sub pd_blogs_for_author {
	my ($app) = @_;
	my $author = $app->{'author'};
	return [] unless $author;
	require MT::Permission;
	my @blogs = ();
	for my $perms (MT::Permission->load({ 'author_id' => $author->id })) {
		next unless $author->is_superuser() || $perms->can_administer_blog;
		my $cfg_key = config_key($perms->blog_id, 'extra');
		my $cfg = load_plugindata($cfg_key);
		next unless ($cfg && $cfg->{'datasource'}
			&& ($cfg->{'datasource'} eq '_pseudo'));
		my $blog = MT::Blog->load($perms->blog_id);
		push(@blogs, { 'id' => $blog->id, 'name' => $blog->name });
	}
	@blogs = sort { $a->{'name'} cmp $b->{'name'} } @blogs;
	return \@blogs;
}

sub uri {
	my ($app) = shift;
	return $app->app_uri(@_);
}

sub db_defs {
	my ($datasource) = @_;
	my %mt_tables = map { $_ => 1 } qw( author blog category comment entry fileinfo ipbanlist log notification permission placement plugindata session tbping template templatemap trackback config );
	if ($mt_tables{$datasource}) {
		return qq{Datasource cannot be one of Movable Type's own tables.};
	} else {
		my $holder = new DatasourceHolder;
		DatasourceHolder::datasource($datasource);
		my $driver = MT::Object->driver;
		my $db_defs = $driver->column_defs($holder);
		if (!$db_defs) {
			return 'Could not load datasource. Check table name.';
		} elsif (!$db_defs->{'id'}) {
			return qq{Table must have a "${datasource}_id" column.};
		} else {
			delete($db_defs->{'id'});
			return $db_defs;
		}
	}
}

package DatasourceHolder;
# dummy object class that we can pass to driver's column_defs() to get
# column defs for an arbitrary datasource
my $datasource;
sub new {
	bless({}, shift);
}
sub properties {
	return { 'datasource' => $datasource };
}
sub datasource {
	$datasource = $_[0];
}
sub column_defs {
# need to define this for sqlite; it's not used for anything we're
# concerned with, so just return a dummy
	return {};
}

1;
